import os
import sys
import typing as t

import pytest
from unfazed.core import Unfazed
from unfazed.test import Requestfactory


@pytest.fixture
async def unfazed() -> t.AsyncGenerator[Unfazed, None]:
    os.environ["UNFAZED_SETTINGS_MODULE"] = "tests.proj.entry.settings"

    path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../proj")

    sys.path.append(path)

    unfazed = Unfazed()
    await unfazed.setup()

    yield unfazed


async def test_api(unfazed: Unfazed) -> None:
    async with Requestfactory(unfazed) as rf:
        resp = await rf.get("/app1/add?a=1&b=2")

        assert resp.status_code == 200
        assert resp.json() == {"result": 3}



import typing as t

import pytest
from pytest import Item
from pytest_asyncio import is_async_test


# dont need decorate test functions with pytest.mark.asyncio
def pytest_collection_modifyitems(items: t.List[Item]) -> None:
    pytest_asyncio_tests = (item for item in items if is_async_test(item))
    session_scope_marker = pytest.mark.asyncio(loop_scope="session")
    for async_test in pytest_asyncio_tests:
        async_test.add_marker(session_scope_marker, append=False)



import os

from taskiq import InMemoryBroker
from taskiq.scheduler.scheduler import TaskiqScheduler
from taskiq_redis import RedisAsyncResultBackend

from unfazed_taskiq import TaskiqAgent
from unfazed_taskiq.settings import UnfazedTaskiqSettings

REDIS_HOST = os.getenv("REDIS_HOST", "redis")

TEMP_SETTINGS = {
    "BROKER": {
        "BACKEND": "taskiq.InMemoryBroker",
        "OPTIONS": {},
    },
    "RESULT": {
        "BACKEND": "taskiq_redis.RedisAsyncResultBackend",
        "OPTIONS": {
            "redis_url": f"redis://{REDIS_HOST}:6379",
        },
    },
    "SCHEDULER": {
        "BACKEND": "taskiq.scheduler.scheduler.TaskiqScheduler",
        "SOURCES": ["taskiq.schedule_sources.LabelScheduleSource"],
    },
}


async def test_agent() -> None:
    agent = TaskiqAgent()

    settings = UnfazedTaskiqSettings.model_validate(TEMP_SETTINGS)
    agent.setup(settings)

    assert isinstance(agent.broker, InMemoryBroker)
    assert isinstance(agent.broker.result_backend, RedisAsyncResultBackend)
    assert isinstance(agent.scheduler, TaskiqScheduler)

    await agent.startup()

    await agent.shutdown()



from unfazed.conf import settings
from unfazed.core import Unfazed
from unfazed.lifespan import BaseLifeSpan

from .base import agent
from .settings import UnfazedTaskiqSettings


class TaskiqLifeSpan(BaseLifeSpan):
    def __init__(self, unfazed: Unfazed) -> None:
        self.unfazed = unfazed
        taskiq_settings: UnfazedTaskiqSettings = settings["UNFAZED_TASKIQ_SETTINGS"]

        agent.setup(taskiq_settings)
        self.agent = agent

    async def on_startup(self) -> None:
        await self.agent.startup()

    async def on_shutdown(self) -> None:
        await self.agent.shutdown()



import typing as t

from pydantic import BaseModel, ConfigDict, Field
from taskiq import ScheduleSource
from unfazed.conf import register_settings


class Broker(BaseModel):
    backend: str = Field(alias="BACKEND")
    options: t.Optional[t.Dict[str, t.Any]] = Field(default=None, alias="OPTIONS")


class Result(BaseModel):
    backend: str = Field(alias="BACKEND")
    options: t.Optional[t.Dict[str, t.Any]] = Field(default=None, alias="OPTIONS")


class Scheduler(BaseModel):
    backend: str = Field(alias="BACKEND")
    sources: t.Optional[t.List[t.Union[str, ScheduleSource]]] = Field(
        default=None, alias="SOURCES"
    )

    model_config = ConfigDict(arbitrary_types_allowed=True)


@register_settings("UNFAZED_TASKIQ_SETTINGS")
class UnfazedTaskiqSettings(BaseModel):
    broker: Broker = Field(alias="BROKER")
    result: t.Optional[Result] = Field(default=None, alias="RESULT")
    scheduler: t.Optional[Scheduler] = Field(default=None, alias="SCHEDULER")



import os

from taskiq_redis import ListRedisScheduleSource

REDIS_HOST = os.getenv("REDIS_HOST", "redis")

UNFAZED_SETTINGS = {
    "LIFESPAN": ["unfazed_taskiq.lifespan.TaskiqLifeSpan"],
    "ROOT_URLCONF": "entry.routes",
    "INSTALLED_APPS": ["app1"],
}


redis_resource = ListRedisScheduleSource(url=f"redis://{REDIS_HOST}:6379")

UNFAZED_TASKIQ_SETTINGS = {
    "BROKER": {
        "BACKEND": "taskiq.InMemoryBroker",
        "OPTIONS": {},
    },
    "RESULT": {
        "BACKEND": "taskiq_redis.RedisAsyncResultBackend",
        "OPTIONS": {
            "redis_url": f"redis://{REDIS_HOST}:6379",
        },
    },
    "SCHEDULER": {
        "BACKEND": "taskiq.scheduler.scheduler.TaskiqScheduler",
        "SOURCES": ["taskiq.schedule_sources.LabelScheduleSource", redis_resource],
    },
}



import asyncio
import os
import sys
import traceback
import warnings


async def main() -> None:
    root_path = os.path.dirname(os.path.abspath(__file__))
    sys.path.append(root_path)
    os.environ.setdefault("UNFAZED_SETTINGS_MODULE", "entry.settings")
    try:
        from unfazed.core import Unfazed
    except ImportError:
        warnings.warn(
            "Could not import unfazed. Did you forget to install it?", stacklevel=2
        )
        sys.exit(1)

    try:
        unfazed = Unfazed()
        await unfazed.setup()

    except Exception as e:
        print(traceback.format_exc())
        warnings.warn(f"An error occurred when init unfazed: {e}", stacklevel=2)
        sys.exit(1)

    await unfazed.execute_command_from_argv()


if __name__ == "__main__":
    asyncio.run(main())



import typing as t

from taskiq import AsyncBroker, ScheduleSource, TaskiqScheduler
from unfazed.conf import settings
from unfazed.utils import import_string

from .settings import UnfazedTaskiqSettings


class TaskiqAgent:
    def __init__(self) -> None:
        self._broker: t.Optional[AsyncBroker] = None
        self._scheduler: t.Optional[TaskiqScheduler] = None

        self._ready = False

    @property
    def broker(self) -> AsyncBroker:
        self.check_ready()
        return t.cast(AsyncBroker, self._broker)

    @property
    def scheduler(self) -> t.Optional[TaskiqScheduler]:
        self.check_ready()
        return self._scheduler

    def check_ready(self) -> None:
        if not self._ready:
            self.setup_from_property()

    def reset(self) -> None:
        self._broker = None
        self._scheduler = None
        self._ready = False

    def setup_from_property(self) -> None:
        taskiq_settings: UnfazedTaskiqSettings = settings["UNFAZED_TASKIQ_SETTINGS"]
        self.setup(taskiq_settings)

    def setup(self, taskiq_settings: UnfazedTaskiqSettings) -> None:
        if self._ready:
            return
        # setup broker
        broker_cls = import_string(taskiq_settings.broker.backend)
        broker_options = taskiq_settings.broker.options

        self._broker = t.cast(AsyncBroker, broker_cls(**broker_options))

        # setup result backend
        if taskiq_settings.result:
            result_cls = import_string(taskiq_settings.result.backend)
            result_options = taskiq_settings.result.options
            self._broker.with_result_backend(result_cls(**result_options))

        # setup scheduler
        if taskiq_settings.scheduler:
            scheduler_cls: t.Type[TaskiqScheduler] = import_string(
                taskiq_settings.scheduler.backend
            )
            scheduler_sources = taskiq_settings.scheduler.sources or []

            sources: t.List[ScheduleSource] = []
            for source in scheduler_sources:
                if isinstance(source, ScheduleSource):
                    sources.append(source)
                else:
                    source_cls = import_string(source)
                    sources.append(source_cls(self._broker))

            self._scheduler = scheduler_cls(broker=self._broker, sources=sources)
        else:
            self._scheduler = None

        self._ready = True

    async def startup(self) -> None:
        await self.broker.startup()
        if isinstance(self.scheduler, TaskiqScheduler):
            await self.scheduler.startup()

    async def shutdown(self) -> None:
        await self.broker.shutdown()
        if isinstance(self.scheduler, TaskiqScheduler):
            await self.scheduler.shutdown()


agent = TaskiqAgent()



import os

from unfazed.utils import import_setting

from .base import TaskiqAgent, agent
from .settings import UnfazedTaskiqSettings


def get_agent() -> TaskiqAgent:
    """
    interface for getting taskiq client

    Example:
        taskiq worker unfazed_taskiq.cli:broker
        taskiq worker unfazed_taskiq.cli:scheduler
    """

    if not os.environ.get("UNFAZED_SETTINGS_MODULE"):
        raise ValueError("UNFAZED_SETTINGS_MODULE is not set")

    # extract settings from unfazed settings
    settings_kv = import_setting("UNFAZED_SETTINGS_MODULE")

    # extract broker settings
    taskiq_settings = settings_kv["UNFAZED_TASKIQ_SETTINGS"]

    settings = UnfazedTaskiqSettings.model_validate(taskiq_settings)

    agent.reset()
    agent.setup(settings)

    return agent


_agent: TaskiqAgent = get_agent()


broker = _agent.broker

scheduler = _agent.scheduler



from unfazed_taskiq import agent


@agent.broker.task
async def add(a: int, b: int) -> int:
    return a + b


@agent.broker.task(schedule=[{"crontab": "*/1 * * * *", "args": [1, 2]}])
async def add_schedule(a: int, b: int) -> int:
    return a + b



import os
import typing as t

import pytest
from taskiq import InMemoryBroker
from taskiq.scheduler.scheduler import TaskiqScheduler
from taskiq_redis import RedisAsyncResultBackend

from unfazed_taskiq.cli import get_agent


@pytest.fixture
def setup_cli() -> t.Generator[None, None, None]:
    os.environ["UNFAZED_SETTINGS_MODULE"] = "tests.proj.entry.settings"

    yield

    del os.environ["UNFAZED_SETTINGS_MODULE"]


@pytest.fixture(autouse=True)
def setup_settings() -> t.Generator:
    from unfazed.conf import settings

    settings.clear()

    yield


@pytest.fixture
def setup_cli2() -> t.Generator[None, None, None]:
    os.environ["UNFAZED_SETTINGS_MODULE"] = "tests.proj.entry2.settings"

    yield

    del os.environ["UNFAZED_SETTINGS_MODULE"]


def test_cli(setup_cli: t.Generator[None, None, None]) -> None:
    agent = get_agent()

    assert isinstance(agent.broker, InMemoryBroker)
    assert isinstance(agent.broker.result_backend, RedisAsyncResultBackend)
    assert isinstance(agent.scheduler, TaskiqScheduler)

    agent.reset()
    assert agent.scheduler is not None


def test_cli_without_env() -> None:
    if "UNFAZED_SETTINGS_MODULE" in os.environ:
        del os.environ["UNFAZED_SETTINGS_MODULE"]

    with pytest.raises(ValueError):
        get_agent()


def test_cli_without_scheduler(setup_cli2: t.Generator[None, None, None]) -> None:
    agent = get_agent()

    assert isinstance(agent.broker, InMemoryBroker)
    assert isinstance(agent.broker.result_backend, RedisAsyncResultBackend)
    assert agent.scheduler is None

    agent.reset()
    assert agent.scheduler is None



UNFAZED_SETTINGS = {
    "LIFESPAN": ["unfazed_taskiq.lifespan.TaskiqLifeSpan"],
    "ROOT_URLCONF": "entry.routes",
}


UNFAZED_TASKIQ_SETTINGS = {
    "BROKER": {
        "BACKEND": "taskiq.InMemoryBroker",
        "OPTIONS": {},
    },
    "RESULT": {
        "BACKEND": "taskiq_redis.RedisAsyncResultBackend",
        "OPTIONS": {
            "redis_url": "redis://redis:6379",
        },
    },
}
